//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     nint.h
// Purpose:    Default Negation Interpretation (Belief Interpretation)
// Last Change:12.03.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	NINT_INCLUDED
#define NINT_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// NINT_BITS: Number of bits in an nint_t value.
//-----------------------------------------------------------------------------

static const int	NINT_BITS = sizeof(unsigned long) * VER_BITS_PER_BYTE;


//=============================================================================
// Type for Default Negation Interpretations:
//=============================================================================

//-----------------------------------------------------------------------------
// Type Definition:
//-----------------------------------------------------------------------------

typedef unsigned long nint_t;

//=============================================================================
// Class (without instances) containing data type functions for nint_t values:
//=============================================================================

class nint_c {

//-----------------------------------------------------------------------------
// Static Member Functions (Class Methods):
//-----------------------------------------------------------------------------

	public:

	// Initialize this module:
	static void	init(int num_crit_neg);

	// Validity check:
	inline static bool_t	valid(nint_t neg_int)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::valid: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::valid: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::valid: NumCritNeg is invalid (2)");
		  return(neg_int <= AllTrue); }

	// Return the interpretation with all negtions interpreted as false:
	inline static nint_t	all_false(void)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::all_false: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::all_false: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::all_false: NumCritNeg is invalid (2)");
		  return(0); }

	// Return the interpretation with all negtions interpreted as true:
	inline static nint_t	all_true(void)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::all_true: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::all_true: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::all_true: NumCritNeg is invalid (2)");
		  return(AllTrue); }

	// Return the next interpretation (for looping over all):
	inline static nint_t	next(nint_t neg_int)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::next: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::next: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::next: NumCritNeg is invalid (2)");
		  CHECK(neg_int <= AllTrue,
			"nint_c::next: neg_int is invalid");
		  CHECK(neg_int != AllTrue,
			"nint_c::next: Already last interpretation");
		  return(neg_int + 1); }

	// Access a bit in the bit array:
	inline static bool_t	is_true(nint_t neg_int, int no)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::value: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::value: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::value: NumCritNeg is invalid (2)");
		  CHECK(no >= 0, "nint_c::value: no is negative");
		  CHECK(no < NumCritNeg,
			"nint_c::value: no larger than NumCritNeg");
		  CHECK(neg_int <= AllTrue,
			"nint_c::value: neg_int is invalid");
		  return((neg_int >> no) & 0x1); }

	// Return the intersection of two interpretations:
	inline static nint_t	intersect(nint_t neg_int1, nint_t neg_int2)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::intersect: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::intersect: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::intersect: NumCritNeg is invalid (2)");
		  CHECK(neg_int1 <= AllTrue,
			"nint_c::intersect: neg_int1 is invalid");
		  CHECK(neg_int2 <= AllTrue,
			"nint_c::intersect: neg_int2 is invalid");
		  return(neg_int1 & neg_int2); }

	// Check whether one interpretation is smaller (a subset) than another:
	inline static nint_t	subset(nint_t neg_int1, nint_t neg_int2)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::subset: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::subset: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::subset: NumCritNeg is invalid (2)");
		  CHECK(neg_int1 <= AllTrue,
			"nint_c::subset: neg_int1 is invalid");
		  CHECK(neg_int2 <= AllTrue,
			"nint_c::subset: neg_int2 is invalid");
		  return((neg_int1 & neg_int2) == neg_int1); }

	// Print an interpretation for the critical default negations:
	static void	print(nint_t neg_int);

	// Query current truth values of facts to determine interpretation:
	static nint_t	get_curr_neg_int(void);

	// Hash function:
	inline static int	hash(nint_t neg_int, int hashsize)
		{ CHECK(NumCritNeg >= 0,
			"nint_c::hash: module not initialized");
		  CHECK(NumCritNeg <= NINT_BITS,
			"nint_c::hash: NumCritNeg is invalid (1)");
		  CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
			"nint_c::hash: NumCritNeg is invalid (2)");
		  CHECK(neg_int <= AllTrue,
			"nint_c::hash: neg_int is invalid");
		  CHECK(hashsize > 0,
			"nint_c::hash: hashsize is not positive");
		  return(neg_int % hashsize); }

//-----------------------------------------------------------------------------
// Static Data Members (Class Attributes):
//-----------------------------------------------------------------------------

	private:

	// Number of critical default negations:
	static int NumCritNeg;

	// Maximal interpretation (all default negations are true)
	static nint_t AllTrue;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

